package org.nakedobjects.nos.store.file;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


public class XmlFile {
    public static final String DEFAULT_ENCODING = "ISO-8859-1";
    private static final String ENCODING_PROPERTY = Configuration.ROOT + "xmlos.encoding";
    private static final String[] escapeString = { "&amp;", "&lt;", "&gt;", "&quot;", "&apos;" };
    private static final String[] specialChars = { "&", "<", ">", "\"", "'" };

    public static String directory() {
        return NakedObjectsContext.getConfiguration().getString(Configuration.ROOT + "xmlos.dir", "xml");
    }

    public static String getValueWithSpecialsEscaped(final String s) {
        String result = s;
        for (int i = 0; i < specialChars.length; i++) {
            String special = specialChars[i];
            int pos = -1;
            while (true) {
                pos = result.indexOf(special, pos + 1);
                if (pos < 0)
                    break;
                result = result.substring(0, pos) + escapeString[i] + result.substring(pos + special.length());
            }
        }
        return result;
    }

    private String charset;
    private File directory;

    public XmlFile() {
        this(directory());
    }

    public XmlFile(final String directory) {
        this.directory = new File(directory);
        if (!this.directory.exists()) {
            this.directory.mkdirs();
        }
        charset = NakedObjectsContext.getConfiguration().getString(ENCODING_PROPERTY, DEFAULT_ENCODING);
    }

    private File file(final String fileName) {
        return new File(directory, fileName + ".xml");
    }

    public void writeXml(final String name, final StringBuffer buf) {
        OutputStreamWriter pw;

        try {
            pw = new OutputStreamWriter(new FileOutputStream(file(name)), charset);
            pw.write("<?xml version=\"1.0\" encoding=\"" + charset + "\" ?>\n");
            pw.write("\n");
            pw.write(buf.toString());
            pw.write("\n");
            pw.close();
        } catch (IOException e) {
            throw new NakedObjectRuntimeException("Problems writing data files", e);
        }
    }

    public boolean parse(final ContentHandler handler, final String fileName) {
        XMLReader parser;

        try {
            parser = XMLReaderFactory.createXMLReader();
        } catch (SAXException e) {
            try {
                parser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
            } catch (SAXException e2) {
                try {
                    parser = XMLReaderFactory.createXMLReader("org.apache.crimson.parser.XMLReaderImpl");
                } catch (SAXException failed) {
                    throw new NakedObjectRuntimeException("Couldn't locate a SAX parser");
                }
            }
        }

        try {
            parser.setContentHandler(handler);
            parser.parse(new InputSource(new InputStreamReader(new FileInputStream(file(fileName)), charset)));

            return true;
        } catch (FileNotFoundException e) {
            return false;
        } catch (IOException e) {
            throw new NakedObjectRuntimeException("Error reading XML file", e);
        } catch (SAXParseException e) {
            throw new NakedObjectRuntimeException("Error while parsing: " + e.getMessage() + " in " + file(fileName) + ")", e);
        } catch (SAXException e) {
            throw new NakedObjectRuntimeException("?? Error parsing XML file " + file(fileName) + " " + e.getClass(), e
                    .getException());
        }
    }

    public void delete(String fileName) {
        file(fileName).delete();
    }

    /**
     * The XML store is deemed to be initialised if the directory used to store the data has no xml files in
     * it.
     */
    public boolean isInitialized() {
        String[] list = directory.list(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.toLowerCase().endsWith(".xml");
            }
        });
        return list.length > 0;
    }
}

// Copyright (c) Naked Objects Group Ltd.
